#!/usr/bin/perl
# 
# Serene Bach - weblog management system
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# [summary]
# Simple installer "install.cgi" is installation tool for Serene Bach.
# This installer is a script to create and check necessary files.
# It is temporary installer for SereneBach Alpha version.
# On beta or release verion of SereneBach, it will be removed.

# [usage]
# Just access it, it will create necessary files and directories.
# If you have an error, please check permission of data directory.
# You need to set permission of data directory at least (usually,
# permission of data directory sets as 777 or 705).

# [after installation]
# Please remove this script on your web server for security reason.
# This script creates the first user who is named "biscuit" and has
# the password as "test".
# So after installation, you can login as "biscuit".  It is strongly
# recommended to change user name and password after login.

# [note]
# This script does not change permission of the following files.
#  - admin.cgi   [755] : admin script
#  - sb.cgi      [755] : main script
#  - mb.cgi      [755] : mobile contents generator
#  - cnt.cgi     [755] : access log collector
#  - install.cgi [755] : this script
#  - init.cgi    [644] : configuration file
#  - addlib.cgi  [644] : indicator for external libs.
#  - log/        [777] : log directory
#  - img/        [777] : image directory
#  - template/   [777] : template parts directory
#  - data/       [777] : data directory
#  - lock/       [777] : lock directory
#  - plugin/     [755] : plugin directory
#  - lib/        [755] : library directory
#  - index.html  [666] : top page html file
#  - style.css   [666] : default style sheet
# Please make sure permissions of those files are correct.

# ==================================================
# // initialize
# ==================================================
use strict;
use lib  qw(. ./lib ./ext);
require 'addlib.cgi' if (-r 'addlib.cgi');
use sb::Driver ();
use sb::Config ();
use sb::Data ();
# ==================================================
# // declaration for local variables
# ==================================================
my @mObjects = (
	'amazon',    'category',  'entry',     'image',     'link',      'message',   
	'plugin',    'session',   'template',  'trackback', 'user',      'weblog',    
);
# ==================================================
# // main routine
# ==================================================
print "Content-Type: text/html\n\n";
eval {
	my $now  = time();
	my $conf = sb::Config->new('config' => 'init.cgi');
	umask(0);
	{ # checking data dir ...
		die("does not exist data dir\n") if (!-d $conf->value('dir_data'));
		die("can not read data dir\n")   if (!-r $conf->value('dir_data'));
		die("can not write data dir\n")  if (!-w $conf->value('dir_data'));
	}
	{ # creating configuration file
		my $file = $conf->value('dir_data') . $conf->value('file_conf');
		if (!-e $file) {
			open(CONFIG,">$file");
			close(CONFIG);
			chmod($conf->value('basic_file_attr'),$file);
		}
		$conf->store if (-e $file);
	}
	# initialize data base
	if ($conf->value('conf_dbtype') eq 'Text') {
		my @hasDetail = ('entry','message','trackback','template','user');
		my $base = $conf->value('dbtxt_data');
		my $suf  = $conf->value('dbtxt_suf');
		my $idx  = $base . $conf->value('dbtxt_ids') . $suf;
		if (!-e $idx) {
			open(INDEX,">$idx");
			close(INDEX);
			chmod($conf->value('basic_file_attr'),$idx);
		}
		die("fail to create index file\n") if (!-e $idx);
		foreach my $name (@mObjects) {
			my $file = $base . $name . $suf;
			if (!-e $file) {
				open(DATA,">$file");
				close(DATA);
				chmod($conf->value('basic_file_attr'),$file);
			}
			die("failt to create data file for $name\n") if (!-e $file);
			if (grep(/^$name$/,@hasDetail)) {
				my $dir = $base . $name;
				mkdir($dir,$conf->value('basic_dir_attr'))  if (!-e $dir);
				die("data dir for $name is not readable\n") if (!-r $dir);
				die("data dir for $name is not writable\n") if (!-w $dir);
			}
		}
		my $lock = $conf->value('dir_lock') . $conf->value('dbtxt_save');
		if (!-e $lock) {
			open(INDEX,">$lock");
			close(INDEX);
			chmod($conf->value('basic_file_attr'),$lock);
		}
		die("fail to create lock file for data base\n") if (!-e $lock);
	}
	# [TODO] other data base
	{ # creating global lock file
		my $lock = $conf->value('dir_lock') . $conf->value('file_lock');
		if (!-e $lock) {
			open(INDEX,">$lock");
			close(INDEX);
			chmod($conf->value('basic_file_attr'),$lock);
		}
		die("fail to create lock file for global lock\n") if (!-e $lock);
	}
	{ # creating file and dir for access log
		my $file = $conf->value('dir_data') . 'log.cgi';
		my $dir  = $conf->value('dir_data') . 'log';
		if (!-e $file) {
			open(INDEX,">$file");
			close(INDEX);
			chmod($conf->value('basic_file_attr'),$file);
		}
		die("fail to create data file for access log\n") if (!-e $file);
		mkdir($dir,$conf->value('basic_dir_attr')) if (!-e $dir);
		die("data dir for access log is not readable\n") if (!-r $dir);
		die("data dir for access log is not writable\n") if (!-w $dir);
		my $lock = $conf->value('dir_lock') . 'cnt';
		if (!-e $lock) {
			open(INDEX,">$lock");
			close(INDEX);
			chmod($conf->value('basic_file_attr'),$lock);
		}
		die("fail to create lock file for access log\n") if (!-e $lock);
	}
	# load sb::Driver
	sb::Driver->new($conf->value('conf_dbtype'));
	# checking or creating the first user ...
	my $admin = sb::Data->load('User','id'=>0);
	$admin = sb::Data->add('User','name'=>'biscuit','stat'=>0) if (!$admin);
	die("failed to create admin data\n") if (!$admin);
	$admin->pass('test');
	sb::Data->update($admin);
	# checking or creating weblog configration ...
	my $weblog = sb::Data->load('Weblog','id'=>0);
	$weblog = sb::Data->add('Weblog') if (!$weblog);
	sb::Data->update($weblog);
	# creating template ...
	my $temp = sb::Data->load('Template');
	if (!$temp) {
		$temp = sb::Data->add('Template',
			'use'   => 1,
			'name'  => 'default',
			'gen'   => $now,
			'mod'   => $now,
			'main'  => &_template_html(),
			'css'   => &_template_css(),
			'entry' => '',
			'info'  => '',
			'files' => '',
		);
		sb::Data->update($temp);
	}
};
&_display_result($@);
exit(0);
# ==================================================
# // sub routines
# ==================================================
sub _display_result {
	my $message = shift;
	if ($message eq '') {
		my $url = sb::Config->get->value('srv_cgi') . sb::Config->get->value('basic_admn');
		$message = sprintf('installation is successful =&gt; <a href="%s">Login</a>',$url);
	} else {
		$message = 'Error : ' . $message;
	}
	print <<"_HTML_RESULT_";
<?xml version=\"1.0\" encoding=\"utf-8\"?>
<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">
<head>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />
<title>simple installer</title>
</head>
<body>
<h1>installation</h1>
<p>$message</p>
</body>
</html>
_HTML_RESULT_
}
sub _template_html {
	return <<'__TEMPLATE_HTML__';
<?xml version="1.0" encoding="{site_encoding}"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="{site_lang}">
<head>
<meta http-equiv="Content-Type" content="text/html; charset={site_encoding}" />
<meta http-equiv="Content-Script-Type" content="text/javascript" />
<meta http-equiv="Content-Style-Type" content="text/css" />
<link rel="stylesheet" href="{site_css}" type="text/css" />
<link rel="alternate" type="application/rss+xml" title="RSS" href="{site_rss}" />
<link rel="alternate" type="application/atom+xml" href="{site_atom}" />
<title>{site_title}</title>
</head>
<body>
<div class="title">
<!-- BEGIN title -->
<h1 id="top">{blog_name}</h1>
<p class="description">{blog_description}</p>
<!-- END title -->
</div>
<div class="body">
<div class="contents">
<!-- BEGIN entry -->
<div class="entry">
<!-- BEGIN sequel -->
<div class="entry_navi">{prev_entry} | <a href="{site_top}">main</a> | {next_entry}</div>
<!-- END sequel -->
<h2><a href="{entry_permalink}">{entry_title}</a></h2>
<div class="entry_author">{entry_date} {entry_time} | posted by {user_name}</div>
<div class="entry_body">{entry_description}</div>
<div class="entry_more">{entry_sequel}</div>
<div class="entry_state"> {category_name} | {comment_num} | {trackback_num}</div>
</div><!--_end_entry_-->
{trackback_auto_discovery}
<!-- END entry -->
<!-- BEGIN comment_area -->
<div class="comment">
<h3 id="comments">Comments</h3>
<!-- BEGIN comment -->
<dl class="body">
<dt>{comment_name} | {comment_time}</dt>
<dd>{comment_description}</dd>
</dl>
<!-- END comment -->
<form action="{site_cgi}" method="post">
<h4>Comment Form</h4>
<dl>
<dt><label for="name" id="labelname">name:</label></dt>
<dd>
<input type="text" tabindex="1" name="name" id="name" value="{cookie_name}" size="20" />
<label for="set_cookie">Cookie</label><input type="checkbox" tabindex="2" name="set_cookie" value="1" id="set_cookie">
</dd>
<dt><label for="email" id="labelemail">email:</label></dt>
<dd><input type="text" tabindex="3" name="email" id="email" value="{cookie_email}" size="30" /></dd>
<dt><label for="url" id="labelurl">url:</label></dt>
<dd><input type="text" tabindex="4" name="url" id="url" value="{cookie_url}" size="30" /></dd>
<dt><label for="description" id="labeldescription">comments:</label></dt>
<dd><textarea tabindex="5" id="description" name="description" rows="8" cols="45"></textarea></dd>
<dt><label for="description" id="labelsubmit">submit</label></dt>
<dd>
<input type="submit" tabindex="6" id="submit" value="submit" />
<input type="reset" tabindex="7" value="reset" />
<input type="hidden" name="entry_id" value="{entry_id}" />
</dd>
</dl>
</form>
</div><!--_end_comment_-->
<!-- END comment_area -->
<!-- BEGIN trackback_area -->
<div class="trackback">
<h3 id="trackback">Trackbacks</h3>
<p class="link">Trackback URL : {trackback_url}</p>
<!-- BEGIN trackback -->
<dl class="body">
<dt>{trackback_title} | {trackback_blog_name} | {trackback_time}</dt>
<dd>{trackback_excerpt}</dd>
</dl>
<!-- END trackback -->
</div><!--_end_trackback_-->
<!-- END trackback_area -->
<!-- BEGIN profile_area -->
<div class="profile">
<h2>{profile_name}'s profile</h2>
<div class="profile_body">{profile_description}</div>
</div><!--_end_profile_-->
<!-- END profile_area -->
<!-- BEGIN page -->
<div class="page">
{prev_page_link} {page_now}/{page_num} {next_page_link}
</div>
<!-- END page -->
</div><!--_end_contents_-->

<div class="navigation">
<!-- BEGIN calendar -->
<dl class="calendar">
<dt id="calendarname">Calendar</dt>
<dd id="calendarlist">{calendar2}
</dd>
</dl>
<!-- END calendar -->
<!-- BEGIN selected_entry -->
<dl class="recententry">
<dt id="newentryname">Selected Entries</dt>
<dd id="newentrylist">{selected_entry_list}
</dd>
</dl>
<!-- END selected_entry -->
<!-- BEGIN recent_comment -->
<dl class="recentcomment">
<dt id="commentname">Recent Comments</dt>
<dd id="commentlist">{recent_comment_list}</dd>
</dl>
<!-- END recent_comment -->
<!-- BEGIN recent_trackback -->
<dl class="recenttrackback">
<dt id="tbname">Recent Trackback</dt>
<dd id="tblist">{recent_trackback_list}</dd>
</dl>
<!-- END recent_trackback -->
<!-- BEGIN category -->
<dl class="category">
<dt id="categoryname">Categories</dt>
<dd id="categorylist">{category_list}</dd>
</dl>
<!-- END category -->
<!-- BEGIN archives -->
<dl class="archives">
<dt id="archivename">Archives</dt>
<dd id="archivelist">{archives_list}</dd>
</dl>
<!-- END archives -->
<!-- BEGIN link -->
<dl class="links">
<dt id="linkname">Links</dt>
<dd id="linklist">{link_list}</dd>
</dl>
<!-- END link -->
<!-- BEGIN profile -->
<dl class="profiles">
<dt id="profilename">Profile</dt>
<dd id="profilelist">{user_list}</dd>
</dl>
<!-- END profile -->
<dl class="extra">
<dt id="extraname">Other</dt>
<dd id="extralist">
<ul>
<li><a href="{site_rss}">RSS1.0</a></li>
<li><a href="{site_atom}">Atom0.3</a></li>
<li>Powered by <a href="http://serennz.cool.ne.jp/sblog/"><strong>sb {script_version}</strong></a></li>
</ul>
</dd>
</dl>
<form method="get" action="{site_cgi}">
<dl class="search">
<dt>Search this site</dt>
<dd>
<input id="search" name="search" size="16" class="form" />
<input type="submit" value="Search" class="button" />
</dd>
</dl>
</form>
</div><!--_end_navigation_-->
<hr />
</div><!--_end_body_-->
<div class="footer">
<address>Copyright &copy; 2004 SimpleBoxes. All Rights Reserved.</address>
<p id="link2top"><a href="#top">page top</a></p>
</div><!--_end_footer_-->
</body>
</html>
__TEMPLATE_HTML__
}
sub _template_css {
	return <<'__TEMPLATE_CSS__';
/* copyright (C) T.Otani */
/* reset elements */
h1, h2, h3, h4, h5, h6 {
	position: static;
	margin: 0px;
	padding: 0px;
}
div, p, hr, blockquote, address, noscript {
	position: static;
	margin: 0px;
	padding: 0px;
	font-style: normal;
	font-weight: normal;
}
ul, ol, dl, dt, dd {
	position: static;
	margin: 0px;
	padding: 0px;
}
pre, table, form {
	position: static;
	margin: 0px;
	padding: 0px;
}
span, img  {
	position: static;
	margin: 0px;
	padding: 0px;
	border: none;
	font-style: normal;
	font-weight: normal;
	border: 0px none #fafaff;
}
em, strong, q, dfn, code, samp, kbd, var, cite, abbr, acronym {
	position: static;
	margin: 0px;
	padding: 0px;
	border: none;
	font-style: normal;
	font-weight: normal;
}
/* basic */
body {
	margin: 0px;
	padding: 0px;
	color: #334;
	background-color: #fff;
}
a {
	text-decoration: underline;
}
a:link {
	color: #66a;
}
a:visited {
	color: #655;
}
a:hover {
	color: #e50;
}
a:active {
	color: #55c;
}
h2,h3,h4 {
	margin: 0px;
	padding: 2px 10px 2px 10px;
	font-family: "Hiragino Kaku Gothic Pro","Osaka",sans-serif;
	line-height: 1.1em;
	font-weight: bold;
	font-size: 110%;
	text-align: left;
	color: #889;
	border-bottom: 1px dotted #bbc;
}
h2 a {
	text-decoration: none;
}
h2 a:link {
	color: #889;
}
h2 a:visited {
	color: #889;
}
h2 a:hover {
	color: #e50;
}
h2 a:active {
	color: #889;
}
pre {
	display: block;
	font-family: monospace;
}
em {
	font-family: "Hiragino Kaku Gothic Pro","Osaka","Arial",sans-serif;
	font-style: normal;
	font-weight: normal;
	border-bottom: 1px dashed #ccf;
}
strong {
	font-family: "Hiragino Kaku Gothic Pro","Osaka","Arial",sans-serif;
	font-weight: bold;
}
blockquote {
	margin: 0px 20px 0px 20px;
	padding: 2px 8px 2px 8px;
	color: #77c;
	line-height: 1.4em;
	border: 1px solid #ddf;
	background-color: #fafaff;
}
blockquote p {
	padding: 0px;
	margin: 0px;
	font-style: italic;
}
p.source {
	margin: 0px 20px 1em 20px;
	padding: 0px;
	font-size: 90%;
	text-align: right;
}
q {
	color: #669;
	font-style: italic;
}
cite {
	font-style: normal;
}
ul {
	padding-left: 2em;
}
ol {
	padding-left: 2em;
}
dl dd {
	padding-left: 2em;
}
del {
	text-decoration: line-through;
}
ins {
	border-bottom: 1px dashed #cfc;
}
/* images */
img.pict {
}
/* title */
div.title {
	margin: 0px;
	padding: 0px;
	color: #fff;
	background-color: #bbd;
}
div.title h1 {
	margin: 0px;
	padding: 10px 10px 5px 10px;
	font-size: 130%;
	font-weight: bold;
	font-family: "Verdana","Osaka",sans-serif;
}
div.title h1 a {
	text-decoration: none;
}
div.title h1 a:link {
	color: #fff;
}
div.title h1 a:visited {
	color: #fff;
}
div.title h1 a:hover {
	color: #fff;
}
div.title h1 a:active {
	color: #fff;
}
div.title p.description {
	margin: 0px;
	padding: 0px 20px 10px 20px;
	color: #ddf;
	font-size: 90%;
}
/* footer */
div.footer {
	margin: 0px;
	padding: 0px;
	text-align: center;
	font-family: "Verdana",sans-serif;
	font-size: 11px;
}
div.footer a {
	text-decoration: none;
}
div.footer address {
	margin: 0px 20px 3px 0px;
}
div.footer p#link2top {
	margin: 0px;
	padding: 10px 20px 10px 20px;
	font-family: "Verdana",sans-serif;
	font-size: 10px;
	text-align: right;
}
div.footer p#link2top:before {
	content: "[";
}
div.footer p#link2top:after {
	content: "]";
}
/* main */
div.contents {
	width: 70%;
	float: left;
	clear: left;
	background-color: #fff;
}
div.page {
	font-size : 90%;
	text-align: center;
}
div.page a {
	text-decoration: none;
}
div.body hr {
	clear: both;
	height: 1px;
	margin: 0px auto;
	padding: 0px;
	color: #fff;
	border-style: none;
	border-top: 1px solid #fff;
}
/* entry */
div.entry {
	margin: 10px 0px 10px 0px;
	padding: 0px 10px 0px 10px;
}
div.entry div.entry_author {
	margin: 0px;
	padding: 2px 10px 2px 10px;
	font-size: 90%;
	text-align: right;
}
div.entry div.entry_author a {
	text-decoration: none;
}
div.entry div.entry_body {
	margin: 0px;
	padding: 15px 20px 5px 20px;
	font-size: 100%;
	line-height: 1.4em;
	color: #445;
}
div.entry div.entry_more {
	margin: 0px;
	padding: 5px 20px 15px 20px;
	font-size: 100%;
	line-height: 1.4em;
	color: #445;
}
div.entry div.entry_state {
	margin: 0px;
	padding: 2px 10px 2px 10px;
	font-size: 90%;
	text-align: right;
}
div.entry div.entry_state a {
	text-decoration: none;
}
div.entry div.entry_navi {
	margin: 0px;
	padding: 0px 10px 10px 10px;
	font-size: 90%;
	text-align: center;
}
div.entry div.entry_navi a {
	text-decoration: none;
}
/* comment */
div.comment {
	margin: 10px 0px 10px 0px;
	padding: 0px 10px 0px 40px;
}
div.comment form {
	margin: 10px 0px;
	padding: 0px;
}
div.comment form dl {
	font-size: 90%;
}
div.comment form dl dt {
	display: block;
	clear: left;
	float: left;
	width: 8em;
	margin: 0px;
	padding: 0px;
	font-family: "Verdana",sans-serif;
	text-align: right;
}
div.comment form dl dt label#labelsubmit {
	display: none;
}
div.comment form dl dd {
	margin: 0px 0px 0px 8.5em;
	padding: 0px;
}
div.comment dl.body {
	margin: 0px;
	padding: 5px 5px 10px 5px;
	line-height: 1.3em;
	font-size: 90%;
}
div.comment dl.body dt {
	margin: 0px;
	padding: 0px;
	color: #889;
	text-align: right;
}
/* trackback */
div.trackback {
	margin: 10px 0px 10px 0px;
	padding: 0px 10px 0px 40px;
}
div.trackback p.link {
	text-align: right;
	font-size: 90%;
}
div.trackback dl.body {
	margin: 0px;
	padding: 5px 5px 10px 5px;
	line-height: 1.3em;
	font-size: 90%;
}
div.trackback dl.body dt {
	margin: 0px;
	padding: 0px;
	color: #889;
	text-align: right;
}
/* profile */
div.profile {
	margin: 10px 0px 10px 0px;
	padding: 0px 10px 0px 10px;
}
div.profile div.profile_body {
	margin: 0px;
	padding: 15px 20px 5px 20px;
	font-size: 100%;
	line-height: 1.4em;
	color: #445;
}
/* navigation */
div.navigation {
	margin-left: 70%;
	padding: 0px 15px 0px 15px;
	font-size: 90%;
}
/* links */
div.navigation dl {
	margin: 15px 0px 15px 0px;
	padding: 0px;
}
div.navigation dl dt {
	margin: 0px;
	padding: 2px 5px 2px 5px;
	font-family: "Verdana",sans-serif;
	font-weight: bold;
	color: #99a;
	border-top: 2px solid #bbc;
	border-bottom: 2px solid #bbc;
	background-color: #fafaff;
}
div.navigation dl dd {
	margin: 0px;
	padding: 0px;
}
div.navigation dl dd ul {
	list-style: none;
	margin: 0px;
	padding: 0px;
}
div.navigation dl dd ul li {
	display: block;
	margin: 0px;
	padding: 2px 10px 2px 10px;
	font-size: 90%;
	color: #668;
	border-bottom: 1px solid #ccd;
	background-color: #fafaff;
}
div.navigation dl dd ul li a {
	text-decoration: none;
}
div.navigation dl.extra dt {
	display: none;
}
div.navigation dl.extra ul {
	margin: 0px;
	padding: 0px;
	list-style: none;
	text-align: center;
}
div.navigation dl.extra a {
	text-decoration: none;
}
div.navigation dl.extra li {
	padding: 3px 0px 3px 0px;
	line-height: 1.3em;
	font-family: "Verdana",sans-serif;
	color: #668;
	border-bottom: 1px none #ccd;
	background-color: transparent;
}
div.navigation dl.search {
	margin: 10px 0px 10px 0px;
	padding: 0px;
	text-align: center;
}
div.navigation dl.search dt {
	font-family: "Verdana",sans-serif;
}
/* calendar */
div.navigation dl.calendar {
	background-color: #fafaff;
	border-bottom: 1px solid #ccd;
}
div.navigation dl.calendar dd {
	margin: 0px;
	padding: 0px;
	text-align: center;
	background-color: #fafaff;
}
div.navigation dl.calendar dd table {
	width: 180px;
	margin: 0px;
	margin-left: auto;
	margin-right: auto;
	padding: 0px;
	border: 0px none;
	border-color: #ccc #333 #333 #ccc;
	background-color: #fafaff;
}
div.navigation dl.calendar dd table td.cell {
	padding: 1px;
	font-family: "Verdana",sans-serif;
	font-size: 10px;
	line-height: 170%;
	color: #bbc;
	text-align: center;
}
div.navigation dl.calendar dd table td.weekday {
	padding: 1px;
	font-family: mono-space;
	font-size: 10px;
	line-height: 170%;
	color: #bbc;
	text-align: center;
}
div.navigation dl.calendar dd table td.cell a {
	margin: -1px;
	padding: 1px;
	text-decoration: none;
	font-weight: bold;
	display: block;
}
div.navigation div.calendar table td.cell span.today {
	color: #b99;
}
div.navigation div.calendar table td.cell a span.today {
	font-weight: bold;
}
div.navigation dl.calendar dd td.calendar_month {
	font-family: "Verdana",sans-serif;
	font-size: 10px;
	text-align: center;
}
div.navigation dl.calendar dd td.calendar_month a {
	text-decoration: none;
}
__TEMPLATE_CSS__
}
